#!/usr/bin/env node
"use strict"
// fixed modules (global variable)
let dependencies = ["bootstrap","rytm-helpers"]
// flexible modules (prompt)
const MODULES_PROMPT = ["gsap","imagesloaded","rytm-webflow","scrollmagic"]

const chalk = require('chalk') // for colors in console.log
const mkdirp = require('mkdirp') // mkdir -p
// const prompt = require('prompt') // prompt (user input)
const shell = require('shelljs') // to execute commands
const fs = require('fs') // file system
const __projectDir = process.cwd()


// welcome
const welcome = () => {
  console.log(chalk.magenta('									'))
  console.log(chalk.magenta('            #############    TM 	'))
  console.log(chalk.magenta('           ################ 		'))
  console.log(chalk.magenta('          ################# 		'))
  console.log(chalk.magenta('         ##### ########### 		'))
  console.log(chalk.magenta('        #####  ######## 			'))
  console.log(chalk.magenta('       #####    ##### 			'))
  console.log(chalk.magenta('      #####      ##### 			'))
  console.log(chalk.magenta('     #####        ##### 			'))
  console.log(chalk.magenta('    #####          ##### 			'))
  console.log(chalk.magenta('									'))
}
const goodbye = () => {
  console.log(chalk.magenta('									'))
  console.log(chalk.magenta('	All good.	One love! '))
  console.log(chalk.magenta('									'))
}

const isModuleInstalled = (name) => {
  return dependencies.indexOf(name) >= 0
}

const getPromptAttrs = () => {
  let props = [
    {
      description: 'Project name',
      name: 'project',
      type: 'string',
      validator: /^[a-zA-Z]+$/,
      warning: 'Project name is not valid, it can only contains letters',
      required: true
    }
  ]
  MODULES_PROMPT.forEach((moduleName) => {
    props.push({
      description: "Include module: " + moduleName,
      name: moduleName,
      type: 'boolean',
      default: true,
    })
  })
  return props
}
/**
 * copy file from template 
 */
const fromTemplate = (from, output, replace) => {
  let content = fs.readFileSync(from, 'utf8')
  replace.forEach((repl) => {
    content = content.split(repl.search).join(repl.replacement)
  })
  fs.writeFileSync(output, content);
}

const buildProject = () => {
  // prompt start
  prompt.start()
  // collect data
  prompt.get(getPromptAttrs(), (err, result) => {
    const projectName = result.project
    let promptDependencies = []
    MODULES_PROMPT.forEach((moduleName) => {
      if (result[moduleName] == true) {
        promptDependencies.push(moduleName)
      }
    })
    dependencies = [...dependencies, ...promptDependencies]
    // # make directories
    mkdirp.sync('public');
    mkdirp.sync('src/scripts');
    mkdirp.sync('src/scss');
    mkdirp.sync('src/svg');
    // ## .gitignore ##
    fs.copyFileSync(
      __dirname + '/templates/_gitignore', 
      __projectDir + "/.gitignore"
    )
    // ## postcss.config.js ##
    fs.copyFileSync(
      __dirname + '/templates/postcss.config.js', 
      __projectDir + "/postcss.config.js"
    )
    // ## webpack.config.js ##
    fs.copyFileSync(
      __dirname + '/templates/webpack.config.js', 
      __projectDir + "/webpack.config.js"
    )
    // ## index.html ##
    fs.copyFileSync(
      __dirname + '/templates/index.html', 
      __projectDir + "/public/index.html"
    )
    // ## SRC files ##
    fs.copyFileSync(
      __dirname + '/templates/src/scss/_buttons.scss', 
      __projectDir + "/src/scss/_buttons.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scss/_typography.scss', 
      __projectDir + "/src/scss/_typography.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scss/_variables.scss', 
      __projectDir + "/src/scss/_variables.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scss/style.scss', 
      __projectDir + "/src/scss/style.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scripts/app.js', 
      __projectDir + "/src/scripts/app.js"
    )
    // ## package.json ##
    fromTemplate(
      __dirname + '/templates/package.json', 
      __projectDir + '/package.json', 
      [
        {
          search: "<%= project_name %>",
          replacement: projectName
        }
      ]
    )
    // ## rytm-webflow dependendent 
    if (isModuleInstalled("rytm-webflow")) {
      mkdirp.sync('src/scripts/controllers');
      mkdirp.sync('src/scripts/views');
      fs.copyFileSync(
        __dirname + '/templates/src/scripts/controllers/Controller.js', 
        __projectDir + "/src/scripts/controllers/Controller.js"
      )
      fs.copyFileSync(
        __dirname + '/templates/src/scripts/views/View.js', 
        __projectDir + "/src/scripts/views/View.js"
      )
      fs.copyFileSync(
        __dirname + '/templates/src/scripts/aswap.js', 
        __projectDir + "/src/scripts/aswap.js"
      )
    }


    // ### npm install
    shell.exec("npm install --save " + dependencies.join(" "))
    shell.exec("npm run build")
    goodbye()

  }) 

}

// welcome msg
welcome()
// build project
buildProject()